/**
 * This file contains part of the source for MGameServer.
 * Copyright (C) 2005 Avo Muromgi
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * 
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
package mgameserver;

import java.io.*;
import java.net.*;

import javax.swing.*;

import mgameserver.games.amazons.*;
import mgameserver.games.clobber.*;
import mgameserver.games.corner.*;
import mgameserver.games.stopgate.*;
import mgameserver.gui.*;
import mgameserver.net.*;
import mgameserver.util.*;

/**
 * Main class for the application.
 * @author Avo Muromgi
 */
public class Program {

    /**
     * Indicates, if we run the program in debug mode.
     */
    public static boolean DEBUG = false;
    
    /**
     * Gives connection with the configuration file.
     */
    public static Resources config;
    private static String configFile = "mgameserver.ini";

    /**
     * Gives connection with the logger.
     */
    public static Logger logger;
    private static String logFile = "mgameserver.log";
    
    /**
     * Creates and returns a new board to draw game state to. The game, that the created board is
     * meant for, is specified in configuration file.
     */
    public static Board getNewGameBoard() {
        int cols = config.getInt("ColCount");
        int rows = config.getInt("RowCount");
        String s = config.getString("Game");
        if (s.equals("stopgate"))
            return new StopGateBoard(new StopGateGame(cols, rows));
        else if (s.equals("amazons"))
            return new AmazonsBoard(new AmazonsGame());
        else if (s.equals("corner"))
            return new CornerBoard(new CornerGame());
        else
            return new ClobberBoard(new ClobberGame(cols, rows));
    }
    
    /**
     * Creates and returns a new client to for communication with client specified by the socket. 
     * The game, that the created client is meant for, is specified in configuration file.
     */
    public static Client getNewClient(Socket socket) throws IOException, Exception {
        String s = config.getString("Game");
        if (s.equals("stopgate"))
            return new StopGateClient(socket);
        else if (s.equals("amazons"))
            return new AmazonsClient(socket);
        else if (s.equals("corner"))
            return new CornerClient(socket);
        else
            return new ClobberClient(socket);
    }
        
    /**
     * Loads applications configuration file and logger, sets up look and feel.
     * @return True, if operation was successful, false otherwise.
     */
    public static boolean startup(){
        try {
            logger = new Logger(true, logFile);
            UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
            config = new Resources(configFile);
            return true;
        } catch (Exception ex) {
            ExceptionHandler.handleException("", ex);
            return false;
        }
    }
    
    /**
     * Shuts down and closes the application.
     */
    public static void shutDown(){
        MainForm.getInstance().dispose();
        System.exit(0);
    }
    
	/**
	 * Programs entry point. 
	 */
	public static void main(String[] args) {
        if (!startup()){
            ExceptionHandler.handleException("Startup failed!", null);
            return;
        }
		MainForm.getInstance().setVisible(true);
	}
}