/**
 * This file contains part of the source for MGameServer.
 * Copyright (C) 2005 Avo Muromgi
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * 
 * See the GNU General Public License for more details.
 */
package mgameserver.games;

/**
 * Abstract base class for games.
 * @author Avo Muromgi
 */
public abstract class Game {

    /**
     * Indicates the integer value for black pieces and squares with black pieces.
     */
    public final static int BLACK = 1;

    /**
     * Indicates the integer value for white pieces and squares with black pieces.
     */
    public final static int WHITE = -BLACK;

    /**
     * Indicates the integer value for empty squares.
     */
    public final static int EMPTY = 0;

    /**
     * Creates a new game with with specified number of columns and rows. 
     */
    public Game(final int colCount, final int rowCount) {
        super();
        this.colCount = colCount;
        this.rowCount = rowCount;
        this.initGame();
    }
    
    /**
     * Gets the last move made.
     */
    public Move getLastMove() {
        return this.lastMove;
    }
    private Move lastMove;

    /**
     * Gets number of columns on the board that the game is played on.
     */
    public int getColCount() {
        return this.colCount;
    }
    private int colCount;

    /**
     * Gets number of rows on the board that the game is played on.
     */
    public int getRowCount() {
        return this.rowCount;
    }
    private int rowCount;

    /**
     * Gets the player, that should make the next move, specified by color  
     */
    public int getTurn() {
        return this.turn;
    }

    /**
     * Sets the player, that should make the next move, specified by color  
     */
    protected void setTurn(final int value) {
        this.turn = value;
    }
    private int turn;

    /**
     * Alters the player who should make the next move.
     */
    protected void switchTurn() {
        this.turn = -this.turn;
    }
    
    /**
     * Gets the state of the square (color of the piece on it or whether it's empty). 
     */
    public int getSqr(final int col, final int row){
        return this.board[col][row];
    }

    /**
     * Sets the state of the square (color of the piece on it or whether it's empty). 
     */
    protected void setSqr(final int col, final int row, final int color) {
        this.board[col][row] = color;
    }
    private int board[][];
    
    /**
     * Initializes the game, sets the board and the pieces to their initial states and sets the game
     * ready to play.
     */
    public void initGame() {
        if (this.board == null)
            this.board = new int[this.colCount + 2][this.rowCount + 2];
        this.setTurn(BLACK);
        this.winner = EMPTY;
        this.lastMove = null;
    }

    /**
     * Gets the color of the winner of the game. <code>EMPTY</code> is returned, when game has not 
     * ended or the result of the game is a draw. 
     */
    public int getWinner(){
        return this.winner;
    }
    
    /**
     * Performs the specified  move in the game.
     * @return True, if move was made and false, when move was illegal.
     */
    public boolean move(final Move move) {
        if (!this.isLegal(move)) {
            this.winner = -this.getTurn();
            return false;
        }
        this.lastMove = move;
        this.switchTurn();
        return true;
    }

    /**
     * Declares the game as ended. The player on turn is declared as a loser.
     */
    public void end() {
        this.setWinner(-this.getTurn());
    }
    
    /**
     * Indicates, if specified move is legal.
     * @return True, if move was legal, false otherwise.
     */
    public abstract boolean isLegal(final Move move);
 
    /**
     * Indicates if game is over.
     */
    public abstract boolean isOver();
    
    /**
     * Sets the color of the winner of the game.  
     */
    protected void setWinner(final int value) {
        this.winner = value; 
    }
    private int winner;
}