/**
 * This file contains part of the source for MGameServer.
 * Copyright (C) 2005 Avo Muromgi
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * 
 * See the GNU General Public License for more details.
 */
package mgameserver.games;

import java.awt.Color;

import mgameserver.*;
import mgameserver.gui.*;
import mgameserver.net.*;
import mgameserver.util.*;

/**
 * Class for conducting games.
 * @author Avo Muromgi
 */
public class Judge extends Thread {

    /**
     * String sent to client, when he has made an illegal move.
     */
    public static String ERROR_MSG = "error";
    
    /**
     * String sent to client, when his opponent has made an illegal move.
     */
    public static String END_MSG = "end";
    
    /**
     * Creates new instance of Judge for conducting specifued games between given clients.
     */
    public Judge(Client player1, Client player2, Game game) {
        super();
        this.player1 = player1;
        this.player2 = player2;
        this.game = game;
        if (player1 == null || player2 == null)
            throw new NullPointerException("One of the players were not evaluated.");
        this.logger = new Logger(
                Program.config.getBoolean("Game.LogToConsole"), 
                Program.config.getString("Game.LogFile"));
        this.logger.setDateTimeLogged(false);
    }

    /**
     * Indicates, if user has exceeded his thinking time limit.
     * NOTE: When a client, who is not playing under supervision of current instance of judge false 
     * is returned. 
     */
    public boolean timeExceeded(Client client) {
        if (client != this.player1 && client != this.player2)
            return false;
        
        int timeLimit = Program.config.getInt("Game.TimeLimit");
        if (timeLimit <= 0)
            return false;
        
        return client.getTimeSpent() > timeLimit * 1000;
    }

    /**
     * Gets the awt color of player 1. 
     */
    public Color getPlayer1Color() {
        return this.player1Color == Game.WHITE ? Color.WHITE : Color.BLACK;
    }
    private int player1Color;
    
    /**
     * Gets the awt color of player 2. 
     */
    public Color getPlayer2Color() {
        return this.player2Color == Game.WHITE ? Color.WHITE : Color.BLACK;
    }
    private int player2Color;

    /**
     * Conducts the games.
     */
    public void run() {
        int gamesPlayed = 0;
        int maxGames = Program.config.getInt("Game.Count");
        int beginner = (this.game.getTurn() == Game.WHITE ? Game.WHITE : Game.BLACK);
        this.player1Color = beginner;
        this.player2Color = -beginner;
        MainForm.getInstance().updateScores();
        
        while (true) {
            this.player1.sendGameStart(
                    this.game.getColCount(), this.game.getRowCount(), player1Color == beginner);
            this.player2.sendGameStart(
                    this.game.getColCount(), this.game.getRowCount(), player2Color == beginner);
            Program.logger.log(this, "Start new game.");
            this.conductGame(this.player1Color, this.player2Color);

            int winner = this.game.getWinner() == this.player1Color ? 1 : 2;
            if (winner == 1) {
                this.player1.sendVictory();
                this.player2.sendLoss();
            } else {
                this.player1.sendLoss();
                this.player2.sendVictory();
            }
            this.logScore(winner, this.player1Color, this.player2Color);
            MainForm.getInstance().updateScores();
            
            if (++gamesPlayed >= maxGames && maxGames > 0)
                return;
            
            if (Program.config.getBoolean("Game.AutoPlay")) {
                this.player1Color = -this.player1Color;
                this.player2Color = -this.player2Color;
            } else {
                switch(NewGameDlg.showNewGameDlg(MainForm.getInstance())) {
                case NewGameDlg.SWITCH_COLORS: 
                    this.player1Color = -this.player1Color;
                    this.player2Color = -this.player2Color;
                    break;
                case NewGameDlg.STOP:
                    Program.shutDown();
                }
            }
            this.game.initGame();
            MainForm.getInstance().getBoardPanel().drawBoard();
            Program.logger.log(this, "initGame");
        }
    }

    private boolean conductMove(Client playerOnTurn, Client playerNotOnTurn) {
        Move move = playerOnTurn.recvMove();
        playerOnTurn.addTimeSpent(System.currentTimeMillis() - this.thinkStart);
        Program.logger.log(this, move.toString() 
                + " by " + playerOnTurn.getName() + " on " + playerOnTurn.getTimeSpent());
        
        if (this.timeExceeded(playerOnTurn)) {
            // Player on turn has exceeded his time limit
            Program.logger.log(this, "Time limit exceeded by " + playerOnTurn.getName());
            this.game.end();
            playerOnTurn.sendError();
            playerNotOnTurn.sendErrorEnd();
            Utils.sleep(Program.config.getInt("Game.PauseBetweenMoves"));
            return false;
        } else { 
            // log move, when user sent it in time
            if (Program.config.getBoolean("Game.LogMoves")) {
                this.logger.log(++this.movesMade + ". " + playerOnTurn.getName());
                this.logger.log(move.toString());
                this.logger.log("Time: " + playerOnTurn.getTimeSpent() + "ms");
                this.logger.addLine();
            }
            
            if (this.game.move(move)) {
                MainForm.getInstance().getBoardPanel().drawBoard();
                Utils.sleep(Program.config.getInt("Game.PauseBetweenMoves"));
                playerNotOnTurn.sendMove(move);
                this.thinkStart = System.currentTimeMillis();
            } else {
                // Player on turn has made an illegal move
                Program.logger.log(this, "Invalid move from " + playerOnTurn.getName());
                this.game.end();
                playerOnTurn.sendError();
                playerNotOnTurn.sendErrorEnd();
                Utils.sleep(Program.config.getInt("Game.PauseBetweenMoves"));
                return false;
            }
        }
        
        return true;
    }
    private long thinkStart = 0;
    private int movesMade = 0;
    
    /**
     * Conducts an entire single game.
     * @param player1Color
     * @param player2Color
     */
    private void conductGame(int player1Color, int player2Color) {
        this.player1.initTimeSpent();
        this.player2.initTimeSpent();
        this.thinkStart = System.currentTimeMillis();
        this.movesMade = 0;
        
        while (!this.game.isOver()) {
            if (this.game.getTurn() == player1Color && !this.game.isOver())
                if (!this.conductMove(this.player1, this.player2))
                    return;
            if (this.game.getTurn() == player2Color && !this.game.isOver())
                if (!this.conductMove(this.player2, this.player1))
                    return;
        }
    }

    private void logScore(int winner, int player1Color, int player2Color) {
        this.logger.addSeparator();
        if (winner == 1) {
            this.player1.addVictory();
            String color = player1Color == Game.BLACK ? "black" : "white"; 
            this.logger.log("Player " + this.player1.getName() + " (" + color + ") won.");
        } else if (winner == 2) {
            this.player2.addVictory();
            String color = player2Color == Game.BLACK ? "black" : "white"; 
            this.logger.log("Player " + this.player2.getName() + " (" + color + ") won.");
        }
        
        this.logger.log(
                this.player1.getName() + " has won " + this.player1.getGamesWon() + " game(s).");
        this.logger.log(
                this.player2.getName() + " has won " + this.player2.getGamesWon() + " game(s).");
        this.logger.addSeparator();
        this.logger.addLine();
        
        Program.logger.log(
                (winner == 1? this.player1 : this.player2).getName() + " has won the game.");
        Program.logger.log("Score - " + this.player1.getName() + " " + this.player1.getGamesWon()
                + " : " + this.player2.getGamesWon() + " " + this.player2.getName());
    }
    
    private Game game;
    private Logger logger;
    private Client player1;
    private Client player2;
} 