/**
 * This file contains part of the source for MGameServer.
 * Copyright (C) 2006 Kristo Tammeoja
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * 
 * See the GNU General Public License for more details.
 */
package mgameserver.games.amazons;

import mgameserver.games.*;

/**
 * Game class for clobber.
 * @author Kristo Tammeoja
 */
public class AmazonsGame extends Game {
    /**
     * Possible horizontal directions for moving. These should be combined with <code>DIR_ROW</code>.
     */
    protected final int[] DIR_COL = new int[]{-1, -1, -1, 0, 0, 1, 1, 1};
    
    /**
     * Possible vertical directions for moving. These should be combined with <code>DIR_COL</code>.
     */
    protected final int[] DIR_ROW = new int[]{-1, 0, 1, -1, 1, -1, 0, 1};

    /**
     * Indicates the integer value for border.
     */
    public final static int BORDER = 2;

    /**
     * Indicates the integer value for arrow.
     */
    public final static int ARROW = 3;

    /**
     * Creates new instance of AmazonsGame.
     */
    public AmazonsGame(){
    	super(10, 10);
    }
    
    public void initGame() {
        super.initGame();
        // borders
        for (int col = this.getColCount() + 1; col >= 0; col--) {
            this.setSqr(col, 0, BORDER);
            this.setSqr(col, this.getRowCount() + 1, BORDER);
        }
        for (int row = this.getRowCount() + 1; row >= 0; row--) {
            this.setSqr(0, row, BORDER);
            this.setSqr(this.getColCount() + 1, row, BORDER);
        }
        // board
        for (int col = this.getColCount(); col >= 1; col--)
            for (int row = this.getRowCount(); row >= 1; row--)
                this.setSqr(col, row, EMPTY);
        // amazons
        this.setSqr(1, 4, WHITE);
        this.setSqr(4, 1, WHITE);
        this.setSqr(7, 1, WHITE);
        this.setSqr(10, 4, WHITE);
        this.setSqr(1, 7, BLACK);
        this.setSqr(4, 10, BLACK);
        this.setSqr(7, 10, BLACK);
        this.setSqr(10, 7, BLACK);
        // beginner is WHITE
        this.setTurn(WHITE);
    }
    
    /* (non-Javadoc)
     * @see mgameserver.game.Game#isOver()
     */
    public boolean isOver(){
        for (int col = this.getColCount(); col >= 1; col--)
            for (int row = this.getRowCount(); row >= 1; row--) {
            	if (this.getSqr(col, row) != this.getTurn()) continue;
            	// player's amazon
                for (int d = 0; d < this.DIR_COL.length; d++)
                	// if amazon has move...
                    if (this.getSqr(col + DIR_COL[d], row + DIR_ROW[d]) == EMPTY) {
                        return false;
                    }
            }
        return true;
    }

    /**
     * NOTE: this should be called from move(Move) only. 
     */
    private void move(int fromCol, int fromRow, int toCol, int toRow, int arrowCol, int arrowRow){
    	int amazon = -this.getTurn(); // super class move has already switched colors
        this.setSqr(fromCol, fromRow, EMPTY);
        this.setSqr(toCol, toRow, amazon);
        this.setSqr(arrowCol, arrowRow, ARROW);
        if (this.isOver())
            this.setWinner(amazon);
    }
    
    /* (non-Javadoc)
     * @see mgameserver.game.Game#move(mgameserver.game.Move)
     */
    public boolean move(final Move move){
        if (super.move(move)) {
        	AmazonsMove m = (AmazonsMove)move;
            this.move(m.getFromCol(), m.getFromRow(),
            		  m.getToCol(), m.getToRow(),
					  m.getArrowCol(), m.getArrowRow()
					 );
            return true;
        } else
            return false;
    }

    public boolean isLegal(Move move) {
    	AmazonsMove m = (AmazonsMove)move;
        
        // right Amazon
        if (this.getSqr(m.getFromCol(), m.getFromRow()) != this.getTurn()) return false;
        
        // Amazon's move must be legal...
        if (!this.isLegalPath(m.getFromCol(), m.getFromRow(),
        		              m.getToCol(), m.getToRow())
		   ) return false;
        
        // ...and arrows path must me empty
        if (!this.isLegalPath(m.getToCol(), m.getToRow(),
                              m.getArrowCol(), m.getArrowRow(),
                              m.getFromCol(), m.getFromRow())
           ) return false;

        return true;
    }
    
    private boolean isOnBoard(int column, int row) {
    	return 1 <= row && row <= this.getRowCount() &&
    	       1 <= column && column <= this.getColCount();
    }
    
    /* For checking Amazon's move */
    private boolean isLegalPath(int startColumn, int startRow, int endColumn, int endRow) {
    	return this.isLegalPath(startColumn, startRow, endColumn, endRow, -1, -1);
    }
    
    /**
     * @param a number
     * @return the sign of the number
     */
    private int sign(int a) {
    	if (a > 0) return 1;
    	else if (a < 0) return -1;
    	else return 0;
    }
    
    /* 
     * For checking Amazon's and arrows move. In case of the arrow, neutral square 
     * represents Amazon's starting square and is not required to be empty.  
     */
    private boolean isLegalPath(int startColumn, int startRow, int endColumn, int endRow,
    		                    int neutralColumn, int neutralRow)
    {
    	// both start and end are on the board, ...
    	if (!this.isOnBoard(startColumn, startRow)) return false;
    	if (!this.isOnBoard(endColumn, endRow)) return false;
    	
    	// ...start and end are different, ...
    	if (startColumn == endColumn && startRow == endRow) return false;
    	
    	int d_column = endColumn-startColumn;
    	int d_row = endRow-startRow;
    	
    	int dir_column = this.sign(d_column);
    	int dir_row = this.sign(d_row);

    	int length = Math.max(Math.abs(d_column), Math.abs(d_row));
    	
    	// ...moving is either orthogonally or diagonally and ...
    	if (startColumn + dir_column * length != endColumn ||
    		startRow + dir_row * length != endRow
    	   ) return false;
    	
    	// ...and the path is empty
    	for (int i = 1; i <= length; i++) {
    		int cur_column = startColumn + dir_column * i; 
    		int cur_row = startRow + dir_row * i;
    		if (EMPTY != this.getSqr(cur_column, cur_row)) {
    			// neutral square (initial position of the amazon when cheking
    			// arrow's path) may be filled
    			if (!(cur_column == neutralColumn && cur_row == neutralRow)) return false;
    		}
    	} // for i
    	
    	return true;
    }
}
