/**
 * This file contains part of the source for MGameServer.
 * Copyright (C) 2005 Avo Muromgi
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * 
 * See the GNU General Public License for more details.
 */
package mgameserver.games.clobber;

import mgameserver.*;
import mgameserver.games.*;

/**
 * Game class for clobber.
 * @author Avo Muromgi
 */
public class ClobberGame extends Game {

    /**
     * Possible horizontal directions for moving. These should be combined with <code>DIR_Y</code>.
     */
    protected final int[] DIR_X = new int[]{0, 1, 0, -1};
    
    /**
     * Possible vertical directions for moving. These should be combined with <code>DIR_X</code>.
     */
    protected final int[] DIR_Y = new int[]{-1, 0, 1, 0};
    
    /**
     * Creates new instance of ClobberGame with default number of columns and rows.
     */
    public ClobberGame(){
        this(Program.config.getInt("ColCount"), Program.config.getInt("RowCount"));
    }
    
    /**
     * Creates new instance of ClobberGame with specified number of columns and rows.
     */
    public ClobberGame(final int colCount, final int rowCount) {
        super(colCount, rowCount);
    }
    
    public void initGame() {
        super.initGame();
        for (int col = this.getColCount() + 1; col >= 0; col--){
            this.setSqr(col, 0, EMPTY);
            this.setSqr(col, this.getRowCount() + 1, EMPTY);
        }
        for (int row = this.getRowCount() + 1; row >= 0; row--){
            this.setSqr(0, row, EMPTY);
            this.setSqr(this.getColCount() + 1, row, EMPTY);
        }
        for (int col = this.getColCount(); col > 0; col--)
            for (int row = this.getRowCount(); row > 0; row--)
                this.setSqr(col, row, ((col + row) & 1) == 0 ? 1 : -1);
    }
    
    /* (non-Javadoc)
     * @see mgameserver.game.Game#isOver()
     */
    public boolean isOver(){
        for (int col = this.getColCount(); col > 0; col--)
            for (int row = this.getRowCount(); row > 0; row--)
                for (int d = this.DIR_X.length - 1; d >= 0; d--)
                    if (this.getSqr(col, row) * this.getSqr(col + DIR_X[d], row + DIR_Y[d]) < 0)
                        return false;
        return true;
    }

    /**
     * NOTE: this should be called from move(Move) only. 
     */
    private void move(final int fromCol, final int fromRow, final int toCol, final int toRow) {
        int turn = -this.getTurn(); // super class move has already switched colors
        this.setSqr(fromCol, fromRow, EMPTY);
        this.setSqr(toCol, toRow, turn);
        if (this.isOver())
            this.setWinner(turn);
    }
    
    /* (non-Javadoc)
     * @see mgameserver.game.Game#move(mgameserver.game.Move)
     */
    public boolean move(final Move move){
        if (super.move(move)) {
            ClobberMove m = (ClobberMove)move;
            this.move(m.getFromCol(), m.getFromRow(), m.getToCol(), m.getToRow());
            return true;
        } else
            return false;
    }

    public boolean isLegal(Move move) {
        ClobberMove m = (ClobberMove)move;
        
        // we don't need to check target square, since if source square is legal, then target square
        // can only be on the border row or column (the row or column, which is "closest" to the 
        // board) and it is guaranteed, that all these cells are empty.
        return !(m.getFromCol() <= 0 || m.getFromCol() > this.getColCount() 
                || m.getFromRow() <= 0 || m.getFromRow() > this.getRowCount()
                || this.getSqr(m.getFromCol(), m.getFromRow()) != this.getTurn() 
                || this.getSqr(m.getToCol(), m.getToRow()) != -this.getTurn()
                || Math.abs(m.getFromCol() - m.getToCol()) 
                + Math.abs(m.getFromRow() - m.getToRow()) != 1);
    }
}