/**
 * This file contains part of the source for MGameServer.
 * Copyright (C) 2005 Avo Muromgi
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * 
 * See the GNU General Public License for more details.
 */
package mgameserver.games.corner;

import mgameserver.Program;
import mgameserver.games.Game;
import mgameserver.games.Move;

/**
 * Game class for from corner to corner.
 * @author Avo Muromgi
 */
public class CornerGame extends Game {

    /**
     * Possible horizontal directions for moving. These should be combined with <code>DIR_Y</code>.
     */
    protected final int[] DIR_X = new int[]{0, 1, 0, -1};
    
    /**
     * Possible vertical directions for moving. These should be combined with <code>DIR_X</code>.
     */
    protected final int[] DIR_Y = new int[]{-1, 0, 1, 0};
    
    /**
     * Creates new instance of ClobberGame with default number of columns and rows and the default 
     * number of columns and rows filled with each players pieces initially.
     */
    public CornerGame(){
        this(Program.config.getInt("ColCount"), Program.config.getInt("RowCount"));
    }
    
    /**
     * Creates new instance of ClobberGame with specified number of columns and rows and the 
     * number of columns and rows filled with each players pieces initially.
     */
    public CornerGame(int colCount, int rowCount) {
        super(colCount, rowCount);
    }

    public void initGame() {
        super.initGame();

        this.piecesPerCol = Program.config.getInt("corner.PiecesPerCol");
        this.piecesPerRow = Program.config.getInt("corner.PiecesPerRow");

        for (int col = this.getColCount(); col > 0; col--)
            for (int row = this.getRowCount(); row > 0; row--)
                this.setSqr(col, row, EMPTY);
        
        for (int col = 1; col <= this.getPiecesPerCol(); col++)
            for (int row = 1; row <= this.getPiecesPerRow(); row++)
                this.setSqr(col, row, BLACK);

        int firstWhiteCol = this.getColCount() - this.getPiecesPerCol() + 1;
        int firstWhiteRow = this.getRowCount() - this.getPiecesPerRow() + 1;
        for (int col = firstWhiteCol; col <= this.getColCount(); col++)
            for (int row = firstWhiteRow; row <= this.getRowCount(); row++) 
                this.setSqr(col, row, WHITE);
    }
    
    /* (non-Javadoc)
     * @see mgameserver.games.Game#isLegal(mgameserver.games.Move)
     */
    public boolean isLegal(Move move) {
        CornerMove m = (CornerMove) move;
        
        // check if source square is on the board
        if (m.getFromCol() <= 0 || m.getFromCol() > this.getColCount() ||
                m.getFromRow() <= 0 || m.getFromRow() > this.getRowCount())
            return false;
        
        // check if source square is of the right color
        if (this.getSqr(m.getFromCol(), m.getFromRow()) != this.getTurn())
            return false;

        // check if target square is on the board
        if (m.getToCol() <= 0 || m.getToCol() > this.getColCount() ||
                m.getToRow() <= 0 || m.getToRow() > this.getRowCount())
            return false;
        
        // check if target square is empty
        if (this.getSqr(m.getToCol(), m.getToRow()) != EMPTY)
            return false;

        // check if was moved horizontally or vertically
        if (m.getFromCol() != m.getToCol() && m.getFromRow() != m.getToRow())
            return false;
        
        // check if the move was not a jump, meaning that the length of the move was 1
        // if the test succeeds there's no need to test the middle square
        if (Math.abs(m.getToCol() - m.getFromCol() + m.getToRow() - m.getFromRow()) == 1)
            return true;
        
        // check if there is a middle square between source and target square 
        // for example, if the target and source columns are 2 and 5 respectively then the middle is
        // between columns 3 and 4 but neither of the columns
        if (m.getFromCol() % 2 != m.getToCol() % 2 || m.getFromRow() % 2 != m.getToRow() % 2)
            return false;

        // check if the middle square between source and target square is not empty
        int middleCol = (m.getToCol() + m.getFromCol()) / 2;
        int middleRow = (m.getToRow() + m.getFromRow()) / 2;
        if (this.getSqr(middleCol, middleRow) == EMPTY)
            return false;
        
        return true; // all checks were successful
    }

    /* (non-Javadoc)
     * @see mgameserver.games.Game#isOver()
     */
    public boolean isOver() {
        boolean result = true;
        for (int col = 1; col <= this.getPiecesPerCol(); col++) {
            for (int row = 1; row <= this.getPiecesPerRow(); row++) 
                if (this.getSqr(col, row) != WHITE) {
                    result = false; // at least one white piece is not in destination
                    break;
                }
            
            if (!result)
                break;
        }
                    
        if (result)
            return true; // all white pieces are in destination area
        
        int firstWhiteCol = this.getColCount() - this.getPiecesPerCol() + 1;
        int firstWhiteRow = this.getRowCount() - this.getPiecesPerRow() + 1;
        for (int col = firstWhiteCol; col <= this.getColCount(); col++)
            for (int row = firstWhiteRow; row <= this.getRowCount(); row++) 
                if (this.getSqr(col, row) != BLACK)
                    return false; // at least one black piece is not in destination
        
        return true; // all black pieces are in destination area
    }

    public void move(final int fromCol, final int fromRow, final int toCol, final int toRow) {
        int turn = -this.getTurn(); // super class move has already switched colors
        this.setSqr(fromCol, fromRow, EMPTY);
        this.setSqr(toCol, toRow, turn);
        if (this.isOver())
            this.setWinner(turn);
    }
    
    /* (non-Javadoc)
     * @see mgameserver.game.Game#move(mgameserver.game.Move)
     */
    public boolean move(final Move move){
        if (super.move(move)) {
            CornerMove m = (CornerMove) move;
            this.move(m.getFromCol(), m.getFromRow(), m.getToCol(), m.getToRow());
            return true;
        } else
            return false;
    }
    
    /**
     * Gets the number of columns with player pieces in the beginning of the game. 
     */
    public int getPiecesPerCol() {
        return this.piecesPerCol;
    }
    private int piecesPerCol;
    
    /**
     * Gets the number of rows with player pieces in the beginning of the game. 
     */
    public int getPiecesPerRow() {
        return this.piecesPerRow;
    }
    private int piecesPerRow;
}
