/**
 * This file contains part of the source for MGameServer.
 * Copyright (C) 2005 Avo Muromgi
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * 
 * See the GNU General Public License for more details.
 */
package mgameserver.games.stopgate;

import mgameserver.*;
import mgameserver.games.*;

/**
 * Game class for Stop-Gate.
 * @author Avo muromgi
 */
public class StopGateGame extends Game {

    /**
     * Value that indicates that square is empty.
     */
    public static final int NONE = 0;
    
    /**
     * Value that indicates that square has bottom part of a piece.
     */
    public static final int BOTTOM = 1;
    
    /**
     * Value that indicates that square has right part of a piece.
     */
    public static final int RIGHT = 2;
    
    /**
     * Value that indicates that square has top part of a piece.
     */
    public static final int TOP = 3;
    
    /**
     * Value that indicates that square has left part of a piece.
     */
    public static final int LEFT = 4;    
    
    public StopGateGame() {
        this(Program.config.getInt("ColCount"), Program.config.getInt("RowCount"));
    }

    public StopGateGame(final int colCount, final int rowCount) {
        super(colCount, rowCount);
    }

    private void move(final int col, final int row) {
        
        // by the time we have reached here, super.move should already have called and therefore
        // turn has already been switched
        final int turn = -this.getTurn();
        final int col_ = col + (turn == BLACK ? 0 : 1);
        final int row_ = row + 1 - col_ + col;

        this.setSqr(col, row, turn);
        this.setSqr(col_, row_, turn);
        this.pieceDir[col][row] = turn == BLACK ? BOTTOM : LEFT;
        this.pieceDir[col_][row_] = turn == BLACK ? TOP : RIGHT;
        
        if (this.isOver())
            this.setWinner(turn);
    }
    
    public boolean move(final Move move) {
        if (super.move(move)) {
            StopGateMove m = (StopGateMove)move;
            this.move(m.getCol(), m.getRow());
            return true;
        } else
            return false;
    }
    
    public boolean isLegal(Move move) {
        StopGateMove m = (StopGateMove)move;
        final int col1 = m.getCol();
        final int row1 = m.getRow();
        final int col2 = col1 + (this.getTurn() == BLACK ? 0 : 1);
        final int row2 = row1 + 1 - col2 + col1;
        
        // col1 <= col2 and row1 <= row2 always stands
        return col1 >= 0 && row1 >= 0 && col2 < this.getColCount() && row2 < this.getRowCount() 
            && this.getSqr(col1, row1) == EMPTY && this.getSqr(col2, row2) == EMPTY;
    }
    
    public boolean isOver() {
        for (int col = this.getColCount() - 1; col >= 0; col--)
            for (int row = this.getRowCount() - 1; row >= 0; row--)
                if (this.isLegal(new StopGateMove(col, row)))
                    return false;
        return true;
    }
    
    /**
     * Indicates the part of the piece that is on the square specified by <code>col</code> and
     * <code>row</code>. 
     * @return Part of the piece, that is on specified square or <code>NONE</code> if square is 
     * empty.
     */
    public int getSqrDir(final int col, final int row) {
        return this.pieceDir[col][row];
    }
    private int[][] pieceDir;

    public void initGame() {
        super.initGame();
        if (this.pieceDir == null)
            this.pieceDir = new int[this.getColCount()][this.getRowCount()];
        for (int i = this.getColCount() - 1; i >= 0; i--)
            for (int j = this.getRowCount() - 1; j >= 0; j--) {
                this.pieceDir[i][j] = NONE;
                this.setSqr(i, j, EMPTY);
            }
    }
}