/**
 * This file contains part of the source for MGameServer.
 * Copyright (C) 2005 Avo Muromgi
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * 
 * See the GNU General Public License for more details.
 */
package mgameserver.net;

import java.io.*;
import java.net.*;
import java.util.*;

import mgameserver.*;
import mgameserver.games.*;

/**
 * Abstract base class for Client.  
 * @author Avo Muromgi
 */
public abstract class Client {

    /**
     * Creates new instance of Client using specified socket.
     * @throws IOException
     * @throws Exception
     */
    public Client(Socket socket) throws IOException, Exception {
        super();
        this.socket = socket;
        this.init();
    }

    /**
     * Sends a game start message to the client. Override this, if you need to handle it.
     * @param cols Number of columns on the game board.
     * @param rows Number of rows on the game board.
     * @param starts Indicates if client should start the game.
     */
    public void sendGameStart(int cols, int rows, boolean starts) {
        // Nothing...
    }

    /**
     * Receives a line from the client. 
     * @throws IOException
     */
    public String recvLine() throws IOException {
         return this.inf.readLine();
    }
    
    /**
     * Receives a Move from the client. Override this, if you need to handle it.
     * @return
     */
    public Move recvMove() {
        return null;
    }

    /**
     * Sends a move to the client. Override this, if you need to handle it.
     */
    public void sendMove(final Move move) {
        // Nothing here...
    }

    /**
     * Sends a string to the client.
     * @param msg
     */
    public void sendString(String s) {
        this.outf.write(s + "\n");
        this.outf.flush();
    }

    /**
     * Sends an end game message to the client, when his opponent made an illegal move or has 
     * exceeded time limit. Override this, if you need to handle it.
     */
    public void sendErrorEnd() {
        // nothing here...
    }
    
    /**
     * Sends an error message to the client, when the client made an illegal move or has exceeded 
     * time limit. Override this, if you need to handle it.
     */
    public void sendError() {
        // nothing here...
    }
    
    /**
     * Sends a victory message to the client, when he has won the game.
     */
    public void sendVictory() {
        // nothing here...
    }
    
    /**
     * Sends a victory message to the client, when he has lost the game.
     */
    public void sendLoss() {
        // nothing here...
    }
    
    /**
     * Gets the name of the client.
     */
    public String getName() {
        return this.name;
    }
    
    /**
     * Sets the name of the client.
     */
    protected void setName(final String value) {
        this.name = value;
    }
    private String name;

    /**
     * Initializes the client.
     * @throws IOException
     * @throws Exception
     */
    protected void init() throws IOException, Exception {
        this.inf = new BufferedReader(new InputStreamReader(this.socket.getInputStream()));
        this.outf = new PrintWriter(this.socket.getOutputStream());
        String line = this.recvLine();
        StringTokenizer st = new StringTokenizer(line);
        if (st.countTokens() != 3) {
            this.isAccepted = false;
            return;
        }
        
        st.nextToken();
        st.nextToken();
        this.setName(st.nextToken()); 
        Program.logger.log(this, "New player " + this.getName());
        this.isAccepted = true;
    }
    
    /**
     * Indicates, if client is accepted by the server. 
     */
    public boolean isAccepted(){
        return this.isAccepted;
    }
    private boolean isAccepted; 
    
    /**
     * Indicates the number of games, that the client has won.
     */
    public int getGamesWon(){
        return this.gamesWon;
    }
    private int gamesWon = 0;
    
    /**
     * Adds a victory to the client wins account.
     */
    public void addVictory(){
        this.gamesWon++;
    }

    /**
     * Gets number of seconds that player has used time for thinking. 
     */
    public long getTimeSpent() {
        return this.timeSpent;
    }
    private long timeSpent;
    
    /**
     * Adds number of seconds to the time that player has used for thinking. 
     */
    public void addTimeSpent(long value) {
        this.timeSpent += value;
    }
    
    /**
     * Sets number of seconds taht player has used time for thinking to zero.
     * NOTE: Use this, when a new game has started.  
     */
    public void initTimeSpent() {
        this.timeSpent = 0;
    }
    
    private BufferedReader inf;
    private PrintWriter outf;
    private Socket socket;
}