/**
 * This file contains part of the source for MGameServer.
 * Copyright (C) 2005 Avo Muromgi
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * 
 * See the GNU General Public License for more details.
 */
package mgameserver.util;

import java.io.*;
import java.util.*;

/**
 * Class for logging events.
 * @author Avo Muromgi
 */
public class Logger {

    private static String addDate(String s){
        int x = s.indexOf("%");
        if (x >= 0) {
            Calendar c = Calendar.getInstance();
            return s.substring(0, x) + c.get(Calendar.YEAR) + c.get(Calendar.MONTH) 
                + c.get(Calendar.DAY_OF_MONTH) + c.get(Calendar.HOUR_OF_DAY) 
                + c.get(Calendar.MINUTE) + c.get(Calendar.SECOND) + s.substring(x + 1, s.length());  
        } else
            return s;
    }
    
    /**
     * Creates a new instance of Logger.
     *
     */
    public Logger() {
        super();
        this.setEnabled(true);
    }
    
    /**
     * Creates a new logger.
     * @param logToConsole Indicates, if events should be logged to the console.
     * @param file The file name, that the events should be logged to. If value is null, then events
     * wont be logged to files. 
     */
    public Logger(boolean logToConsole, String file) {
        this();
        this.setLogFile(file);
        this.setLogToConsole(logToConsole);
        this.log("");
        this.setDateTimeLogged(true);
    }

    /**
     * Sets a file name, that the events should be logged to.
     */
    public void setLogFile(String file) {
        try { 
            if (this.file != null)
                this.outf.close();
            this.file = file;
            if (this.file != null) {
                this.file = addDate(this.file);
                this.outf = new PrintWriter(new FileWriter(this.file, true), true);
            }
        } catch(IOException ex) {
            ExceptionHandler.handleException("", ex);
        }
    }
    
    /**
     * Logs a message. Given object class name is logged aswell.
     */
    public void log(Object o, String message) {
        if (!this.isEnabled())
            return;
        String msg = "";
        if (this.getName() != null)
            msg += this.getName() + "    ";
        if (this.isDateTimeLogged())
            msg += Calendar.getInstance().getTime() + " - ";
        if (o != null)
            msg += o.getClass().getName() + ": ";
        msg += message;
        
        if (this.getLogToConsole())
            System.out.println(msg);
        if (this.outf != null)
            this.outf.println(msg);
    }
    
    /**
     * Logs a message. 
     */
    public void log(String message){
        this.log(null, message);
    }
    
    /**
     * Adds a separator line to the log output.
     */
    public void addSeparator(){
        this.log(SEPARATOR);
    }
    private static final String SEPARATOR = 
        "=================================================================================";
    
    /**
     * Adds an empty line to the log output.
     */
    public void addLine(){
        this.log("");
    }

    /**
     * Indicates, if date and time should also be logged, while logging events.
     */
    public boolean isDateTimeLogged() {
        return this.dateTimeLogged;
    }

    /**
     * Sets, whether date and time should also be logged, while logging events.
     */
    public void setDateTimeLogged(boolean dateTimeLogged) {
        this.dateTimeLogged = dateTimeLogged;
    }
    private boolean dateTimeLogged;

    /**
     * Gets the name of the logger.
     */
    public String getName() {
        return this.name;
    }

    /**
     * Sets the name of the logger.
     */
    public void setName(String name) {
        this.name = name;
    }
    private String name;

    /**
     * Indicates, if events should be logged to the console as well. 
     */
    public boolean getLogToConsole() {
        return this.logToConsole;
    }

    /**
     * Sets, whether events should be logged to the console as well. 
     */
    public void setLogToConsole(boolean logToConsole) {
        this.logToConsole = logToConsole;
    }
    private boolean logToConsole; 
    
    /**
     * Indicates, if logger is enabled.
     */
    public boolean isEnabled() {
        return this.enabled;
    }

    /**
     * Sets, whether logger is enabled.
     */
    public void setEnabled(boolean enabled) {
        this.enabled = enabled;
    }
    private boolean enabled;
    
    private String file;
    private PrintWriter outf;
}
